#define TWO_WAY

MODULE agrif_lim2_update
   !!======================================================================
   !!                       ***  MODULE agrif_lim2_update ***
   !! Nesting module :  update surface ocean boundary condition over ice
   !!                   from a child grif
   !! Sea-Ice model  :  LIM 2.0 Sea ice model time-stepping
   !!======================================================================
   !! History :  2.0   !  04-2008  (F. Dupont)  initial version
   !!            3.2   !  09-2010  (R. Benshila, C. Herbaut) update and EVP
   !!----------------------------------------------------------------------
#if defined key_agrif && defined key_lim2
   !!----------------------------------------------------------------------
   !!   'key_lim2'  :                                 LIM 2.0 sea-ice model
   !!   'key_agrif' :                                 AGRIF library 
   !!----------------------------------------------------------------------
   !!   agrif_update_lim2  : update sea-ice model on boundaries or total
   !!                        sea-ice area
   !!----------------------------------------------------------------------
   USE ice_2
   USE dom_ice_2
   USE sbc_oce
   USE dom_oce
   USE agrif_oce
   USE agrif_ice 

   IMPLICIT NONE
   PRIVATE

   PUBLIC agrif_update_lim2

   !!----------------------------------------------------------------------
   !! NEMO/NST 3.2 , LOCEAN-IPSL (2010)
   !! $Id$
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE agrif_update_lim2 ( kt )
      !!----------------------------------------------------------------------
      !!   *** ROUTINE agrif_update_lim2 ***
      !!----------------------------------------------------------------------
      INTEGER, INTENT(in) :: kt
      !!
      REAL(wp), DIMENSION(:,:)  , ALLOCATABLE :: zvel
      REAL(wp), DIMENSION(:,:,:), ALLOCATABLE :: zadv, zsadv
      !!----------------------------------------------------------------------
      !
      IF((Agrif_NbStepint() .NE. (Agrif_irhot()-1)).AND.(kt /= 0)) RETURN

      Agrif_UseSpecialValueInUpdate = .TRUE.
      Agrif_SpecialValueFineGrid = 0.

# if defined TWO_WAY
      ALLOCATE( zvel(jpi,jpj), zadv(jpi,jpj,7), zsadv(jpi,jpj,42) )

      IF( MOD(nbcline,nbclineupdate) == 0) THEN
         CALL Agrif_Update_Variable( zadv , adv_ice_id , procname = update_adv_ice  )
         CALL Agrif_Update_Variable( zsadv, sadv_ice_id, procname = update_sadv_ice )
         CALL Agrif_Update_Variable( zvel , u_ice_id   , procname = update_u_ice    )
         CALL Agrif_Update_Variable( zvel , v_ice_id   , procname = update_v_ice    )
      ELSE
         CALL Agrif_Update_Variable( zadv , adv_ice_id , locupdate=(/0,2/), procname = update_adv_ice  )
         CALL Agrif_Update_Variable( zsadv, sadv_ice_id, locupdate=(/0,2/), procname = update_sadv_ice ) 
         CALL Agrif_Update_Variable( zvel , u_ice_id   , locupdate=(/0,1/), procname = update_u_ice    )
         CALL Agrif_Update_Variable( zvel , v_ice_id   , locupdate=(/0,1/), procname = update_v_ice    )
      ENDIF

      DEALLOCATE( zvel, zadv, zsadv )
# endif
      !
   END SUBROUTINE agrif_update_lim2


   SUBROUTINE update_adv_ice( tabres, i1, i2, j1, j2, before )
      !!-----------------------------------------------------------------------
      !!                         *** ROUTINE update_adv_ice ***
      !!-----------------------------------------------------------------------
      INTEGER, INTENT(in) :: i1, i2, j1, j2
      REAL(wp), DIMENSION(i1:i2,j1:j2,7), INTENT(inout) :: tabres
      LOGICAL, INTENT(in) :: before
      !!
      INTEGER :: ji, jj
      REAL(wp) :: zrhox, zrhoy
      REAL(wp) :: z1_area
      !!-----------------------------------------------------------------------
      !
      IF( before ) THEN
         zrhox = Agrif_Rhox()
         zrhoy = Agrif_Rhoy()
         DO jj=j1,j2
            DO ji=i1,i2
               tabres(ji,jj, 1) = frld  (ji,jj) * area(ji,jj)
               tabres(ji,jj, 2) = hicif (ji,jj) * area(ji,jj)
               tabres(ji,jj, 3) = hsnif (ji,jj) * area(ji,jj)
               tabres(ji,jj, 4) = tbif  (ji,jj,1) * area(ji,jj)
               tabres(ji,jj, 5) = tbif  (ji,jj,2) * area(ji,jj)
               tabres(ji,jj, 6) = tbif  (ji,jj,3) * area(ji,jj)
               tabres(ji,jj, 7) = qstoif(ji,jj) * area(ji,jj)
            END DO
         END DO
         tabres = zrhox * zrhoy * tabres
      ELSE
         DO jj=j1,j2
            DO ji=i1,i2
               z1_area = 1. / area(ji,jj) * tms(ji,jj)
               frld  (ji,jj)   = tabres(ji,jj, 1) * z1_area
               hicif (ji,jj)   = tabres(ji,jj, 2) * z1_area
               hsnif (ji,jj)   = tabres(ji,jj, 3) * z1_area
               tbif  (ji,jj,1) = tabres(ji,jj, 4) * z1_area
               tbif  (ji,jj,2) = tabres(ji,jj, 5) * z1_area
               tbif  (ji,jj,3) = tabres(ji,jj, 6) * z1_area
               qstoif(ji,jj)   = tabres(ji,jj, 7) * z1_area
            END DO
         END DO
      ENDIF

   END SUBROUTINE update_adv_ice

#if defined key_lim2_vp
   SUBROUTINE update_u_ice( tabres, i1, i2, j1, j2, before )
      !!-----------------------------------------------------------------------
      !!                        *** ROUTINE update_u_ice ***
      !!-----------------------------------------------------------------------
      INTEGER, INTENT(in) :: i1, i2, j1, j2
      REAL(wp), DIMENSION(i1:i2,j1:j2), INTENT(inout) :: tabres
      LOGICAL, INTENT(in) :: before
      !!
      INTEGER :: ji, jj
      REAL(wp) :: zrhoy
      !!-----------------------------------------------------------------------
      !
      IF( before ) THEN
         zrhoy = Agrif_Rhoy()
         DO jj=MAX(j1,2),j2
            DO ji=MAX(i1,2),i2
               tabres(ji,jj) = e2f(ji-1,jj-1) * u_ice(ji,jj)
            END DO
         END DO
         tabres = zrhoy * tabres
      ELSE
         DO jj= MAX(j1,2),j2
            DO ji=MAX(i1,2),i2
               u_ice(ji,jj) = tabres(ji,jj) / (e2f(ji-1,jj-1))
               u_ice(ji,jj) = u_ice(ji,jj) * tmu(ji,jj)
            END DO
         END DO
      ENDIF
      !
   END SUBROUTINE update_u_ice


   SUBROUTINE update_v_ice( tabres, i1, i2, j1, j2, before )
      !!-----------------------------------------------------------------------
      !!                    *** ROUTINE update_v_ice ***
      !!-----------------------------------------------------------------------
      INTEGER, INTENT(in) :: i1,i2,j1,j2
      REAL(wp), DIMENSION(i1:i2,j1:j2),  INTENT(inout) :: tabres
      LOGICAL, INTENT(in) :: before
      !!
      INTEGER :: ji, jj
      REAL(wp) :: zrhox
      !!-----------------------------------------------------------------------
      !
      IF( before ) THEN
         zrhox = Agrif_Rhox()
         DO jj=MAX(j1,2),j2
            DO ji=MAX(i1,2),i2
               tabres(ji,jj) = e1f(ji-1,jj-1) * v_ice(ji,jj)
            END DO
         END DO
         tabres = zrhox * tabres
      ELSE
         DO jj=j1,j2
            DO ji=i1,i2
               v_ice(ji,jj) = tabres(ji,jj) / (e1f(ji-1,jj-1))
               v_ice(ji,jj) = v_ice(ji,jj) * tmu(ji,jj)
            END DO
         END DO
      ENDIF
      !
   END SUBROUTINE update_v_ice
#else
   SUBROUTINE update_u_ice( tabres, i1, i2, j1, j2, before )
      !!-----------------------------------------------------------------------
      !!                        *** ROUTINE update_u_ice ***
      !!-----------------------------------------------------------------------
      INTEGER, INTENT(in) :: i1, i2, j1, j2
      REAL(wp), DIMENSION(i1:i2,j1:j2), INTENT(inout) :: tabres
      LOGICAL, INTENT(in) :: before
      !!
      INTEGER :: ji, jj
      REAL(wp) :: zrhoy
      !!-----------------------------------------------------------------------
      !
      IF( before ) THEN
         zrhoy = Agrif_Rhoy()
         DO jj=MAX(j1,2),j2
            DO ji=MAX(i1,2),i2
               tabres(ji,jj) = e2u(ji,jj) * u_ice(ji,jj)
            END DO
         END DO
         tabres = zrhoy * tabres
      ELSE
         DO jj=MAX(j1,2),j2
            DO ji=MAX(i1,2),i2
               u_ice(ji,jj) = tabres(ji,jj) / (e2u(ji,jj))
               u_ice(ji,jj) = u_ice(ji,jj) * tmu(ji,jj)
            END DO
         END DO
      ENDIF
      ! 
   END SUBROUTINE update_u_ice


   SUBROUTINE update_v_ice( tabres, i1, i2, j1, j2, before )
      !!-----------------------------------------------------------------------
      !!                    *** ROUTINE update_v_ice ***
      !!-----------------------------------------------------------------------
      INTEGER, INTENT(in) :: i1,i2,j1,j2
      REAL(wp), DIMENSION(i1:i2,j1:j2),  INTENT(inout) :: tabres
      LOGICAL, INTENT(in) :: before
      !!
      INTEGER :: ji, jj
      REAL(wp) :: zrhox
      !!-----------------------------------------------------------------------
      !
      IF( before ) THEN
         zrhox = Agrif_Rhox()
         DO jj=MAX(j1,2),j2
            DO ji=MAX(i1,2),i2
               tabres(ji,jj) = e1v(ji,jj) * v_ice(ji,jj)
            END DO
         END DO
         tabres = zrhox * tabres
      ELSE
         DO jj=j1,j2
            DO ji=i1,i2
               v_ice(ji,jj) = tabres(ji,jj) / (e1v(ji,jj))
               v_ice(ji,jj) = v_ice(ji,jj) * tmv(ji,jj)
            END DO
         END DO
      ENDIF
      !
   END SUBROUTINE update_v_ice
#endif

   SUBROUTINE update_sadv_ice( tabres, i1, i2, j1, j2, before )
      !!-----------------------------------------------------------------------
      !!                     *** ROUTINE update_sadv_ice ***
      !!-----------------------------------------------------------------------
      !!
      INTEGER, INTENT(in) :: i1, i2, j1, j2
      REAL(wp), DIMENSION(i1:i2,j1:j2,42), INTENT(inout) :: tabres
      LOGICAL, INTENT(in) :: before
      !!
      INTEGER :: ji, jj
      REAL(wp) :: zrhox, zrhoy, zarea, z1_area
      !!-----------------------------------------------------------------------
      !
      IF( before ) THEN
         zrhox = Agrif_Rhox()
         zrhoy = Agrif_Rhoy()
         DO jj=j1,j2
            DO ji=i1,i2
               zarea = area(ji,jj)
               tabres(ji,jj, 1) = sxice (ji,jj) * zarea
               tabres(ji,jj, 2) = syice (ji,jj) * zarea  
               tabres(ji,jj, 3) = sxxice(ji,jj) * zarea
               tabres(ji,jj, 4) = syyice(ji,jj) * zarea
               tabres(ji,jj, 5) = sxyice(ji,jj) * zarea
               tabres(ji,jj, 6) = sxa   (ji,jj) * zarea
               tabres(ji,jj, 7) = sya   (ji,jj) * zarea
               tabres(ji,jj, 8) = sxxa  (ji,jj) * zarea
               tabres(ji,jj, 9) = syya  (ji,jj) * zarea
               tabres(ji,jj,10) = sxya  (ji,jj) * zarea
               tabres(ji,jj,11) = sxsn  (ji,jj) * zarea
               tabres(ji,jj,12) = sysn  (ji,jj) * zarea
               tabres(ji,jj,13) = sxxsn (ji,jj) * zarea
               tabres(ji,jj,14) = syysn (ji,jj) * zarea
               tabres(ji,jj,15) = sxysn (ji,jj) * zarea
               tabres(ji,jj,16) = sxc0  (ji,jj) * zarea
               tabres(ji,jj,17) = syc0  (ji,jj) * zarea
               tabres(ji,jj,18) = sxxc0 (ji,jj) * zarea
               tabres(ji,jj,19) = syyc0 (ji,jj) * zarea
               tabres(ji,jj,20) = sxyc0 (ji,jj) * zarea
               tabres(ji,jj,21) = sxc1  (ji,jj) * zarea
               tabres(ji,jj,22) = syc1  (ji,jj) * zarea
               tabres(ji,jj,23) = sxxc1 (ji,jj) * zarea
               tabres(ji,jj,24) = syyc1 (ji,jj) * zarea
               tabres(ji,jj,25) = sxyc1 (ji,jj) * zarea
               tabres(ji,jj,26) = sxc2  (ji,jj) * zarea
               tabres(ji,jj,27) = syc2  (ji,jj) * zarea
               tabres(ji,jj,28) = sxxc2 (ji,jj) * zarea
               tabres(ji,jj,29) = syyc2 (ji,jj) * zarea
               tabres(ji,jj,30) = sxyc2 (ji,jj) * zarea
               tabres(ji,jj,31) = sxst  (ji,jj) * zarea
               tabres(ji,jj,32) = syst  (ji,jj) * zarea
               tabres(ji,jj,33) = sxxst (ji,jj) * zarea
               tabres(ji,jj,34) = syyst (ji,jj) * zarea
               tabres(ji,jj,35) = sxyst (ji,jj) * zarea
            END DO
         END DO
         tabres = zrhox * zrhoy * tabres
      ELSE
         DO jj=j1,j2
            DO ji=i1,i2
               z1_area = 1. / area(ji,jj) * tms(ji,jj)
               sxice (ji,jj) =  tabres(ji,jj, 1) * z1_area
               syice (ji,jj) = tabres(ji,jj, 2) * z1_area  
               sxxice(ji,jj) = tabres(ji,jj, 3) * z1_area
               syyice(ji,jj) = tabres(ji,jj, 4) * z1_area
               sxyice(ji,jj) = tabres(ji,jj, 5) * z1_area
               sxa   (ji,jj) = tabres(ji,jj, 6) * z1_area
               sya   (ji,jj) = tabres(ji,jj, 7) * z1_area
               sxxa  (ji,jj) = tabres(ji,jj, 8) * z1_area
               syya  (ji,jj) = tabres(ji,jj, 9) * z1_area
               sxya  (ji,jj) = tabres(ji,jj,10) * z1_area
               sxsn  (ji,jj) = tabres(ji,jj,11) * z1_area
               sysn  (ji,jj) = tabres(ji,jj,12) * z1_area
               sxxsn (ji,jj) = tabres(ji,jj,13) * z1_area
               syysn (ji,jj) = tabres(ji,jj,14) * z1_area
               sxysn (ji,jj) = tabres(ji,jj,15) * z1_area
               sxc0  (ji,jj) = tabres(ji,jj,16) * z1_area
               syc0  (ji,jj) = tabres(ji,jj,17) * z1_area
               sxxc0 (ji,jj) = tabres(ji,jj,18) * z1_area
               syyc0 (ji,jj) = tabres(ji,jj,19) * z1_area
               sxyc0 (ji,jj) = tabres(ji,jj,20) * z1_area
               sxc1  (ji,jj) = tabres(ji,jj,21) * z1_area
               syc1  (ji,jj) = tabres(ji,jj,22) * z1_area
               sxxc1 (ji,jj) = tabres(ji,jj,23) * z1_area
               syyc1 (ji,jj) = tabres(ji,jj,24) * z1_area
               sxyc1 (ji,jj) = tabres(ji,jj,25) * z1_area
               sxc2  (ji,jj) = tabres(ji,jj,26) * z1_area
               syc2  (ji,jj) = tabres(ji,jj,27) * z1_area
               sxxc2 (ji,jj) = tabres(ji,jj,28) * z1_area
               syyc2 (ji,jj) = tabres(ji,jj,29) * z1_area
               sxyc2 (ji,jj) = tabres(ji,jj,30) * z1_area
               sxst  (ji,jj) = tabres(ji,jj,31) * z1_area
               syst  (ji,jj) = tabres(ji,jj,32) * z1_area
               sxxst (ji,jj) = tabres(ji,jj,33) * z1_area
               syyst (ji,jj) = tabres(ji,jj,34) * z1_area
               sxyst (ji,jj) = tabres(ji,jj,35) * z1_area
            END DO
         END DO
      ENDIF

   END SUBROUTINE update_sadv_ice

#else
CONTAINS
   SUBROUTINE agrif_lim2_update_empty
      !!---------------------------------------------
      !!   *** ROUTINE agrif_lim2_update_empty ***
      !!---------------------------------------------
      WRITE(*,*)  'agrif_lim2_update : You should not have seen this print! error?'
   END SUBROUTINE agrif_lim2_update_empty
#endif
END MODULE agrif_lim2_update
